#ifdef  ENCRYPT
# ifdef AUTHENTICATION
#  ifdef IDEA_ENCRYPTION
#include <arpa/telnet.h>
#include <stdio.h>
#ifdef  __STDC__
#include <stdlib.h>
#endif
 
#include "encrypt.h"
#include "key-proto.h"
#include "misc-proto.h"
#include "idea.h"

extern encrypt_debug_mode;

#define IDEA_IN	0
#define IDEA_OUT	1

char got_session_key=0;
char need_kick_start=0;
struct IdeaCfbContext context[2];

int Server;

/*
 * This is called once when the program starts
 */

void idea_init(server)
        int server;
{
  Server = server;  
}

/*
 * This is called when we're supposed to start encrypting in a
 * particular direction.
 */
int idea_start(dir, server)
        int dir;
        int server;
{
  if (encrypt_debug_mode)
    printf("called start\n");

  if (got_session_key)
    return 0;
  else
  {
    need_kick_start++;
    return 7;
  }
}

void idea_encrypt(s, c)
        unsigned char *s;
        int c;
{
/*
 * XXX: Uh, large values of c will smash the stack...
 */
  unsigned char outbuf[1024];

#if 0
  if (encrypt_debug_mode)
    printf("Encrypting %d bytes...",c);
#endif

  ideaCfbEncrypt(&context[IDEA_OUT],s,outbuf,c);
  memcpy(s,outbuf,c);
#if 0
  if (encrypt_debug_mode)
    printf("done\n",c);
#endif
}

int idea_decrypt(c)
int c;
{
/*
 * XXX: -1 is supposed to make us "back up". What the hell THAT
 * is supposed to mean is anyone's guess. -1 will make this
 * blow up, probably.
 */

  unsigned char in,out;
  static unsigned char last = 0;
  static unsigned int putback = 0;

  if (c<0)
  {
    putback++;
    return 0;
  }
  if (putback)
  {
    putback=0;
    return last;
  }

  in=c;

#if 0
  if(encrypt_debug_mode)
    printf("decrypting...");
#endif
  ideaCfbDecrypt(&context[IDEA_IN],&in,&out,1);
#if 0
  if(encrypt_debug_mode)
    printf("done\n");
#endif
  last=out;
  return out;
}

/*
 * This is called by the auth layer once per session.
 * We init _here_ because we need the key to init.
 */
void idea_session(key, server)
        Session_Key *key;
        int server;
{
  unsigned char keybuf[16];

  unsigned char msg[] = { IAC, SB, TELOPT_ENCRYPT, ENCRYPT_IS, ENCTYPE_IDEA,
    IAC, SE };
#define IDEA_IS_LEN 7

/*
 * Chuck if it's not our kind of key. A better one will come along.
 */
  if (key->type != SK_IDEA)
    return ;

  if (encrypt_debug_mode)
  {
    char j;
    printf("*** Got session key: %d bytes\n",key->length);
    for(j=0;j<key->length;j++)
      printf("%02x:",*((key->data)+j));
    printf("\n");
  }

  got_session_key=1;

/*
 * use memset & min of 16, length to insure against being passed a short key
 */
  memset(keybuf,0,16);
  memcpy(keybuf,key->data,min(16,key->length));

  ideaCfbInit(&context[IDEA_IN],keybuf);
  ideaCfbInit(&context[IDEA_OUT],keybuf);

/*
 * Here's the tricky part.
 *
 * If start() was called _before_ we had a session key,
 * it can only be because the other end wants to start
 * encrypting as soon as possible. We told him we couldn't
 * do it yet and noted that we'd have to kickstart encryption
 * when we had our private key. So we know that the other
 * end wants us to decrypt.
 *
 * If we had something to _negotiate_, it would be easy:
 * we would simply call idea_start() ourselves. But
 * idea_start() in this case doesn't really _do_ anything,
 * so nothing in encrypt.c will _notice_ that we've done
 * so. So we have to cheat and make encrypt.c call
 * start, notice that start has returned 0 and start
 * encrypting. The way to do this is to send an encrypt_is
 * to the other side. He'll reply and we'll both turn it on.
 *
 * *whimper*
 */

  if (need_kick_start)
  {
    need_kick_start=0;
    net_write(msg,IDEA_IS_LEN);
  }

  if (encrypt_debug_mode)
    printf("Exiting idea_session()\n");
}

#endif /* IDEA_ENCRYPTION */
#endif /* AUTHENTICATION */
#endif /* ENCRYPTION */
